_ = require 'lodash'
uuid = require 'uuid'
redis = require 'fakeredis'
MessageSent = require '..'

describe 'MessageSent', ->
  beforeEach ->
    @redisKey = uuid.v1()
    @uuidAliasResolver = resolve: sinon.stub().yields null, 'sender-uuid'
    options = {
      firehoseClient: redis.createClient(@redisKey)
      @uuidAliasResolver
    }

    dependencies = {@request}

    @sut = new MessageSent options, dependencies
    @cache = redis.createClient @redisKey

  describe '->do', ->
    context 'when given a valid message', ->
      beforeEach (done) ->
        @cache.subscribe 'sender-uuid', (error) =>
          done error

      beforeEach ->
        @cache.once 'message', (channel, @message) =>

      beforeEach (done) ->
        request =
          metadata:
            responseId: 'its-electric'
            auth:
              uuid: 'sender-uuid'
              token: 'sender-token'
            toUuid: 'receiver-uuid'
            fromUuid: 'sender-uuid'
            messageType: 'sent'
          rawData: '{"devices":"*"}'

        @sut.do request, (error, @response) => done error

      it 'should return a 204', ->
        expectedResponse =
          metadata:
            responseId: 'its-electric'
            code: 204
            status: 'No Content'

        expect(@response).to.deep.equal expectedResponse

      it 'should publish the message to redis', (done) ->
        _.delay =>
          expect(@message).to.deep.equal '{"devices":"*"}'
          done()
        , 100

    context 'when given a valid message with an alias', ->
      beforeEach (done) ->
        @cache.subscribe 'sender-uuid', (error) =>
          done error

      beforeEach ->
        @cache.once 'message', (channel, @message) =>

      beforeEach (done) ->
        request =
          metadata:
            responseId: 'its-electric'
            auth:
              uuid: 'sender-uuid'
              token: 'sender-token'
            toUuid: 'receiver-uuid'
            fromUuid: 'muggle-mouth'
            messageType: 'sent'
          rawData: '{"devices":"*"}'
        @uuidAliasResolver.resolve.yields null, 'sender-uuid'
        @sut.do request, (error, @response) => done error

      it 'should return a 204', ->
        expectedResponse =
          metadata:
            responseId: 'its-electric'
            code: 204
            status: 'No Content'

        expect(@response).to.deep.equal expectedResponse

      it 'should publish the message to redis', (done) ->
        _.delay =>
          expect(@message).to.deep.equal '{"devices":"*"}'
          done()
        , 100
